import React, { useMemo } from 'react';
import { Marks } from './Marks';
import { scaleSqrt, max, scaleOrdinal } from 'd3';
import { ColorLegend } from './ColorLegend';

const fadeOpacity = 0.25;
const maxRadius = 20;

const longitude = d => d.longitude;
const latitude = d => d.latitude;
const name = d => d.name;
const sizeValue = d => d.yield_upper;

export const BubbleMap = ({
  nuclearData,
  mapData,
  setClickedValue,
  attributePurpose,
  attributeType,
  setHoveredValue,
  hoveredValue,
  clickedValue,
  width,
  height,
  brushExtent,
  year
}) => {
  
  const sizeScale = useMemo(
    () =>
      scaleSqrt()
        .domain([0, max(nuclearData, sizeValue)])
        .range([3, maxRadius]),
    [nuclearData, sizeValue, maxRadius]
  );

  const colorValue = d => d.country;
  const colorLegendLabel = 'Country';
  const colorScale = scaleOrdinal()
    .domain(nuclearData.map(colorValue))
    .range([
      '#BD2D28',
      '#E6842A',
      '#E3BA22',
      '#5C8100',
      '#0F8C79',
      '#356B90',
      '#684664'
    ]);

  const filteredNuclearData = nuclearData.filter(
    d => hoveredValue === colorValue(d)
  );

  const filteredBrushData = brushExtent
    ? nuclearData.filter(d => {
        const date = year(d);
        return date > brushExtent[0] && date < brushExtent[1];
      })
    : nuclearData;

  return (
    <svg width={width} height={height}>
      <Marks
        mapData={mapData}
        nuclearData={filteredNuclearData}
        longitude={longitude}
        latitude={latitude}
        name={name}
        sizeScale={sizeScale}
        sizeValue={sizeValue}
        colorScale={colorScale}
        colorValue={colorValue}
        onClick={setClickedValue}
        purpose={attributePurpose}
        type={attributeType}
      />
      <g opacity={hoveredValue ? fadeOpacity : 1}>
        <Marks
          mapData={mapData}
          nuclearData={filteredBrushData}
          longitude={longitude}
          latitude={latitude}
          name={name}
          sizeScale={sizeScale}
          sizeValue={sizeValue}
          colorScale={colorScale}
          colorValue={colorValue}
          onClick={setClickedValue}
          purpose={attributePurpose}
          type={attributeType}
        />
      </g>
      <g transform={`translate(10, 40)`}>
        <text x={35} y={-17} className="axis-label" textAnchor="middle">
          {colorLegendLabel}
        </text>
        <ColorLegend
          colorScale={colorScale}
          tickSpacing={20}
          tickSize={9}
          tickTextOffset={12}
          onHover={setHoveredValue}
          hoveredValue={hoveredValue}
          fadeOpacity={fadeOpacity}
        />
      </g>
      <g transform={`translate(15, 40)`}>
        <text x={1} y={294} className="info">
          {clickedValue ? 'Name: ' + clickedValue.name : ''}
        </text>
        <text x={1} y={306} className="info">
          {clickedValue ? 'Year: ' + clickedValue.year : ''}
        </text>
        <text x={1} y={318} className="info">
          {clickedValue ? 'Type: ' + clickedValue.type : ''}
        </text>
        <text x={1} y={330} className="info">
          {clickedValue ? 'Magnitude: ' + clickedValue.yield_upper : ''}
        </text>
      </g>
    </svg>
  );
};
